@@ -17,3 +17,10 @@
  */
 
 #include "buffer.hpp"
+
+namespace obby {
+
+const unsigned long PROTOCOL_VERSION = 8ul;
+
+}
+
